/****************************************************************************************************/
/* Microchip/Circuit Cellar																			*/
/*																									*/
/* FileName:		Oximeter.c																		*/
/* Dependencies:	p30f2012.h																		*/
/* Processor:		dsPIC30F2012																	*/
/* Compiler:		MPLAB C30 v3																	*/	
/* Author:			MT2278																			*/
/* Date:			January 17, 2008																*/
/*Notes:			Period to Frequency Conversion/Frequency Method									*/
/*																									*/
/*																									*/
/*HARDWARE CONFIGURATION:																			*/
/*PORT I/O																							*/
/* B0	VIRon		Signal to H-Bridge used to control IR Source On/OFF {1/0}						*/
/* B1	VRon		Signal to H-Bridge used to control RED Source On/OFF (1/0)						*/
/* B2`	*SS1		Chip Select to the Microchip MPC4822 SPI DAC device.							*/
/* B3	R/W			Read/Write line to the Hantronix Inc. HDM08216H-3-S00S LCD Display.				*/
/* B4	D4			LCD Data bus bit 4 (4 Bit Bus)													*/
/* B5	D5			LCD Data bus bit 5 (4 Bit Bus)													*/
/* B6	D6			LCD Data bus bit 6 (4 Bit Bus)													*/
/* B7	D7			LCD Data bus bit 7 (4 Bit Bus)													*/
/* B8	--			Not Used																		*/
/* B9	PULSE		Diagnostic Output used during debug												*/
/*																									*/
/* RC13	U1ATX		Connects to T1IN of MAX232 device. Provides RS232 data output for system.		*/	
/* RC14 U1ARX		Connects to R1OUT of MAX232 device. Provides RS232 data input for system.		*/
/*																									*/
/* D8	IC1			Connects to the Output of the TAOS TSL235R LTF device to measure freq. input	*/	
/* D9	SW1			Momentary Switch used to switch between data output modes.						*/
/*																									*/
/* F3	SDO1		SPI Serial Data Output pin that connects to the Microchip MCP4822 DAC device	*/
/* F4	E			Enable Line to the display. State is Toggled over 100us.						*/
/* F5	RS			Select Data or Control Register in the display. Command/Data = 0/1.				*/
/* F6	SCK1		SPI Clock Output to Microchip MCP4822 SPIDAC device.							*/
/*																									*/			
/*																									*/
/*TIMER1 MODULE																						*/
/*Purpose: 	Timebase - sets sample rate																*/
/*Type:		A																						*/
/*Source:	Internal Clock FCY = XTFREQ*PLLMODE/4													*/
/*								where, 	PLLMODE = 16	 											*/
/*										XTFREQ = 7372800 HZ											*/
/*										FCY = 29,491,200											*/
/*Prescaler: 256																					*/
/*PR1 = 0x240 (576 dec)																				*/
/*Interrupt Enabled to provide a period of 5ms. Data sampled every other period (Ts = 10ms)			*/	
/*																									*/
/*																									*/	
/*TIMER2																							*/
/*Purpose: 	Timebase - sets measurement resolution													*/		
/*Type:		A																						*/
/*Source	Internal Clock FCY = XTFREQ*PLLMODE/4													*/	
/*								where, 	PLLMODE = 16 												*/
/*										XTFREQ = 7372800 HZ											*/
/*										FCY = 29,491,200											*/
/*Prescaler: 1																						*/	
/*Free Running (Not Interrupted)																	*/
/*Each timer tick (33.8084201ns) is used to measure the period of TAOS LTF device. The inverse of	*/
/*this measurement is proportional to the Intensity of the Source being measured.					*/
/*																									*/
/*																									*/
/*INPUT CAPTURE MODULE																				*/
/*Purpose:	Setup to trigger interrupt at IC1 pin on rising edge of TAOS device output.By timing	*/
/*			two periods from the device, the period of the TAOS output can be measured using the 	*/
/*			free running timer T2.																	*/
/*Source:	IC1 pin (external)																		*/
/*Number:	One Capture per interrupt																*/
/*Mode:		Captures every RISING edge																*/
/*Interrupt Enabled to measure instances reletive to T2 timer (reference)							*/
/*																									*/
/*																									*/
/*SPI MODULE																						*/		
/*Purpose:	Send Command Word to Microchip MCP4822 Digital to Analog Converter. Controls LED		*/
/*			intensity.																				*/
/*Framed:	Disabled																				*/
/*Data:		16 bit																					*/
/*Mode:		Active clock to idle clock,active state hi, Master Mode,								*/
/*			Secondary Prescaler: /1																	*/
/*			Primary Prescaler: /4																	*/		
/*ClkFrq:	7.372800MHz																				*/
/*																									*/
/*																									*/
/*																									*/
/*UART MODULE																						*/
/*Purpose:	Send pulse data out of system for further diagnostics									*/	
/*Baudrate:	115200																					*/
/*Data:		8																						*/
/*Parity:	N																						*/
/*Stopbit:	1																						*/
/*Flow:		No Flow Control																			*/	
/****************************************************************************************************/


/*Headers*/	
#include "p30F2012.h"
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <float.h>
#include <errno.h>
#include <limits.h>
#include <setjmp.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <assert.h>
#include <yvals.h>


/*Definitions*/
#define XTFREQ          7372800         			//On-board Crystal frequency
#define PLLMODE         16               			//On-chip PLL setting
#define FCY             XTFREQ*PLLMODE/4        	//Instruction Cycle Frequency
#define BAUDRATE         115200     				//Tested 2400,4800,9600,19.2K,38.4K,57.6K,115.2K
#define BRGVAL          ((FCY/BAUDRATE)/16)-1 
#define BigString		64
#define E				_RF4						//Enable Pin for Display					
#define	RS				_RF5						//Register Select Pin for Display
#define RW				_RB3						//Read/Write line for Display (R/*W)
#define CS				_RB2						//Chip Select for DAC (Active LOW)
#define VRon			_RB1						//Turn Red LED ON
#define VIRon			_RB0						//TURN IRed LED ON
#define PULSE			_RB9						//Toggle this output every 10ms.
#define ADJUST_PERIOD	10000						//Number of Samples before LED Tuneup
#define Ts				0.010						//Sample Time
#define Fo				3.0							//LP Filter Cutoff Frequency
#define Fh				10.0						//HP Filter Cutoff Frequency (Differentiator)
#define PI				3.14159265					//Constant			
#define BAND			2							//Intensity Band for Auto Adjustment
#define STEPSIZE		1							//Large Incremental Step size for LED Intensity
#define PeakDetTC		.4							//Peak Detector Time Constant in seconds
#define PeakDetDB		10							//Peak Detector Deadband value
#define	MEDIAN_SIZE		20							//Size of Median Filter

 
/*Variables*/
char TValueB;										//Timer Subroutine Counters
unsigned int TValueW;
unsigned int TValue;
long timer;
int RDaqVal,IRDaqVal;								//RED and IRED D/A Values
int STATE;											//State Counter for T1 Interrupt routine
char message[40];									//Buffer array for COM port
char LCDstr[20];									//Buffer Array for Liquid Crystal Display
char buf[5];										//Scratch Buffer Array for itoa routine
char r,c;											//ROW and Column for cursor 
int MEASURE;										//State Counter for the IC1 interrupt handler
int GoodR,GoodIR;									//Valid Intensity Data has been acquired from IR handler
unsigned int T0;									//Initial Timer values acquired in IC1 interrupt handler
unsigned int RPeriod;								//Period measured from TAOS device for RED led.
unsigned int RFreq[5];								//Frequency Array of time shifted samples for RED led.
unsigned int RFilti,R_DCi;							//LP and DC Tracking Filter integer outputs (RED Led)
float R_DC,R_DCM1;									//M and M-1 Outputs for DC Tracking Filter (RED Led)
unsigned int IRPeriod;								//Period measured from TAOS device for IR led.
unsigned int IRFreq[5];								//Frequency Array of time shifted samples for IR led.	
unsigned int IRFilti,IR_DCi;						//LP and DC Tracking Filter integer outputs (IR Led)
float IR_DC,IR_DCM1;								//M and M-1 Outputs for DC Tracking Filter (IR Led)
float RFilt[5];										//M through M-5 Outputs for L.P. Filter (RED Led)
float IRFilt[5];									//M through M-5 Outputs for L.P. Filter (IR Led)
float dRFilt,dRFiltM1;								//M and M-1 Outputs for H.P. Filter (RED Led)
float dIRFilt,dIRFiltM1;							//M and M-1 Outputs for H.P. Filter (IR Led)
float Wo;											//Angular Cutoff Frequency for L.P. Filter
float Wh;											//Angular Cutoff Frequency for H.P. Filter
float K;											//DC Tracking Filter Gain
float C0,K1,K2;										//H.P. Filter Coefficients
float C1,M,N,N0,N1,N2,N3,N4,D0,D1,D2,D3,D4;			//L.P. Filter Coefficients
float Tc;											//Time Constant(s) for output to decay to 1/2 peak value
float DecayFactor;									//Peak Detector Decay Factor
float SMA,SMAM1;									//SMA Filter M and M-1 output for Heartbeat
float RSum,Rrms,IRSum,IRrms,Rx,SPO2;				//Variables for calculating SPO2
int SP2[MEDIAN_SIZE];								//Array of (MEDIAN_SIZE) SPO2 values
float SPO2ave,SPO2aveM1;							//Simple Moving Average M and M-1 output for SPO2
int X[MEDIAN_SIZE];									//Array of (MEDIAN_SIZE) Heartbeat values
int SMAindex;										//Index for SMA Filter
int dindex,SampleSize;								//Counter for tweeking LED intensity
float RDiff;										//AC calculated from RED Led
float IRDiff;										//AC calculated from IR Led	
float Peak;											//Peak Follower Output
int SampleCounter,Beats,BPM;						//Variables used in calculating HeartRate
int	PDDBCntr;										//Counter to setup Deadband after Peak Detection.
int MeasureFlag;									//Flag used to determine when to calculate BPM and SPO2.
int ADJ_FLAG;										//Flag used by Auto Adjust routine
int PrintFlg;										//Flag used to determine when to output data
int MeasurementDone;								//Flag used by IC1 routine to indicate when measurement is done.
int	indx;											//Attay Indexer
int	SW1cnt;											//State Counter for Input of SWITCH #1
int	SWState;										//State if Switch
int DataSet;										//Indicates which data set to output to Com Port
unsigned int RDat,R_DCDat;
unsigned int IRDat,IR_DCDat;
const char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";


/*Routines Prototypes*/
void Preamble(void);								//Routine to display initial message
void Write2LCD(void);								//Writes to LCD HeartRate and SPO2
void PrintIRDat(void);								//Sends, IRDiff, dIRFilt and Peak signals to serial port
void PrintDat(void);								//Sends, RDat, R_DCDat, IR_Dat and IR_DCDat signal to serial port.
void PrintResults(void);							//Sends BPM, SPO2 to the serial port
void auto_adj(void);								//Routine to provide auto adjustment of the IR LED when called.
void IDBrightLED(void);								//Writes DAC commands, Tests brightness of Each LED
void SPIInit(void);									//Initialized SPI port for use with MicroChip MCP4822 DAC chip.
void SPIOut(unsigned int);							//Send data out SPI port.
void DACOut(char,unsigned int);						//Send RED/IR Led Intensity Command via DAC.
char *strrev(char *);								//Reverses the characters in a string
char *itoa(int, char *, int);						//Converts (int) to string
void UARTinit(int);									//Initialized RS232 port for 8-N-1 with a Baud rate of 115200.
void SendChar(char);								//Sends a single character out the Serial Port
void SendString(char []);							//Sends a NULL terminated string out the Serial Port.
void T5us(void);									//5us Timer
void T10us(void);									//10us Timer
void T50us(void);									//50us Timer
void T100us(void);									//100us Timer 
void T1ms(void);									//1ms Timer	
void T5ms(void);									//5ms Timer	
void T15ms(void);									//15ms Timer	
void Initdisp(void);								//Routine to initialize Display on powerup
void toggleE(void);									//Toggle E-input of display
void DispDat(unsigned char);						//Routine to display Data Character on display
void LCDwrite(char, char, char *);					//Write string to display at ROW{0,1}, COLUMN(0-7)
void DispCmd(unsigned char);						//Writes a command to the display
void cursor(unsigned char, unsigned char);			//Places cursor at specified location ROW(0,1) COLUMN(0-7)
void curOFF(void);									//Turns cursor OFF
void curON(void);									//Turns cursor ON
void clear(void);									//Routine to CLEAR the display
void blinkON(void);									//Routine to make the CURSOR BLINK
void blinkOFF(void);								//Routine to turn off BLINKING CURSOR
void InitT1(void);									//Routine to initalized timer T1
void DisableT1(void);								//Routine to DISABLE timer T1
void EnableT1(void);								//Routine to ENABLE timer T1
void InitT2(void);									//Routine to Iniialize Timer T2	
void DisableT2(void);								//Routine to DISABLE timer T2
void EnableT2(void);								//Routine to ENABLE timer T2
void InitIC1(void);									//Routine to Initialized Input Capture IC1
void DisableIC1(void);								//Routine to Disable Input Capture IC1
void __attribute__((interrupt)) _T1Interrupt(void);
void __attribute__((interrupt)) _IC1Interrupt(void);


/*Main*/
int main(void)
	{
	ADPCFG = 0x3FF;									//Make analog pins digital 
	LATB = 0b1000000100;							//Configure Port B
	TRISB = 0xFC00;									
	LATD = 0x200;									//Configure Port D
	TRISD = 0xFFFF;									
	LATF = 0;										//Configure Port F
	TRISF = 0x87;									
	UARTinit(BRGVAL);								//Initialize the UART for serial communications
	Initdisp();										//Initialize Display
	SPIInit();										//Initialize SPI port
	InitT1();										//Initialize Counter T1
	STATE = -1;										//Initialize STATE counter
	ADJ_FLAG = 0;									//Initialize Flags
	GoodR = 0;
	GoodIR = 0;
	RDaqVal = 3800;
	IRDaqVal = 1900;								//Set DAC commands
//	RDaqVal = 1800;
//	IRDaqVal = 600;									//Set DAC commands
	dindex = 0;										//Initialize counter for tweeking LED intansity
	SampleSize = ADJUST_PERIOD/100;

	Wo = 2.*PI*Fo;									//Calculate Angular cutoff frequency for L.P. Filter
	Wh = 2.*PI*Fh;									//Calculate Angular cutoff frequency for H.P. Filter
	K = 256.;										//DC Tracking Filter Time Constant

	C0 = tanf(Ts*Wh/2.);							//H.P. Frequency Constant
	K1 = 1.0/(1.0 + C0);							//Calculate Coefficients for High Pass Filter
	K2 = (1. - C0)/(1.0 + C0);

	C1 = 1./tanf(Ts*Wo/2.);							//L.P. Frequency Constant
	M = 256.*(float)sqrt(26.)/500.;					//Calculate Coefficients for Low Pass Filter
	N = 34118./10000.;
	N0 = 1.;
	N1 = 4.;
	N2 = 6.;
    N3 = 4.;
	N4 = 1.;
	D0 = C1 * ( C1 *( C1 *(C1 + M) + N) + M) + 1.;
	D1 = - ( C1* (C1* (C1* (4.*C1 + 2.* M) ) - 2. * M) - 4.);
	D2 = C1 * C1 * (6. * C1 * C1 - 2. * N) + 6.;
	D3 = -(C1* (C1* (C1* (4. *C1 - 2. *M) ) +2. *M) -4. );
	D4 = C1 * ( C1 * ( C1 * ( C1 - M ) + N) - M) + 1.;

	Tc = PeakDetTC;									//Iniialize Peak Follower Parameters
//	DecayFactor = 0.993;							//Calculate Decay Factor for Peak Follower 
	DecayFactor = powf(0.5,(4.*PeakDetDB*Ts*Tc));			   
	for(indx=0;indx<5;indx++)						//Setup Initial State of L.P. Filter
		{
		RFreq[indx] = RFilt[indx] = 0.0;
		IRFreq[indx] = IRFilt[indx] = 0.0;
		}
	dIRFiltM1 = 0;									//Setup Initial State of H.P. Filter
	for(Beats = 0;Beats < MEDIAN_SIZE;Beats++)		//Initialize SMA Filter Arrays for:
		{
		X[Beats] = 0;								//Hearbeat
		SP2[Beats] = 0;								//SPO2
		}
	SMAM1 = 0;										//Set inital conditions for SMA filter.
	Beats = 0;										//Initialize X[] and SP2[] indexer
	SampleCounter = 0;								//Initialize Heartbeat sample counter
	MeasureFlag = -1;								//Set MeasureFlag
	Peak = 0.0;										//Initialize Peak Follower
	PDDBCntr = 0;									//Clear Peak Follower Deadband Counter
	RSum = 0;										//Initialized Accumulator for RMS calculation
	IRSum = 0;
	SPO2 = 0.;										//Initialize SPO2 and initial state.
	SPO2aveM1 = 0.;
	PrintFlg = 0;									//Initialized Print Flag
	DataSet = 1;									//Initial State of Data_Set
	SW1cnt = 0;
	SWState = 0;
	Preamble();										//Call to routine for initial display
	while(RFreq[0] < 100 && IRFreq[0] < 100)		//Wait for finger to be placed into probe
		{
		clear();									//Clear diaplay,	
		strcpy(LCDstr,"Waiting");					//Write message
		LCDwrite(0,0,LCDstr);
		for(timer=0;timer<500000;timer++);			//Wait for a period,		
		clear();									//clear display,
		strcpy(LCDstr," Waiting");					//Write message,
		LCDwrite(1,0,LCDstr);
		for(timer=0;timer<500000;timer++);			//Wair for a period,		
		IDBrightLED();								//Measure TAOS LTF sensor to measure LED intensities
		}
	clear();

	for(timer=0;timer<1000000;timer++);				//Delay a spell.	
	DACOut(1,RDaqVal);								//Set RED Intensity		
	while(1)
		{
		if(_RD9 == 0 && !SWState)					//If SW1 is pressed and SWState Flag = 0
			SWState = -1;							//Set the flag.
		else if(_RD9 == 1 && SWState)				//Otherwise, if SW1 is not pressed and SWState==-1,
			{
			DataSet++;								//Increment the DataSet counter.
			if(DataSet == 3)						//If the counter has reached 3,
				DataSet = 0;						//loop back to 0.
			SWState = 0;							//Reset the SWState flag to 0.
			}
		else										//Otherwise,
			SWState = 0;							//Reset the SWState flag to 0.

		if(STATE == 0xFFFF)							//STATE: -1
			{
			auto_adj();								//Set the intensity of the IR LED close to that of the RED.
			LATB &= 0xFFFE;							//Turn OFF RED Led
			LATB &= 0xFFFD;							//Turn OFF IRED Led
			STATE++;								//Increment STATE counter
			EnableT1();								//Enable timer T1
			}

		if(GoodR == -1 && GoodIR == -1)				//If measurement are valid, Process!!!
			{		
			RDat = RFilti;							//Save a copy of RFilti for output (Diagnostics)
			R_DCDat = R_DCi;						//Save a copy of R_DCi for output  (Diagnostics)

			IRDat = IRFilti;						//Save a copy of IRFilti for output (Diagnostics)
			IR_DCDat = IR_DCi;						//Save a copy of IR_DCi for output  (Diagnostics)

			if (dIRFilt <= Peak)					//Check the output of the H.P. Filter (DIFFERENTIATOR),
				{									//If <= Peak value,
  				Peak = dIRFilt;						//Save the Peak Value of the Derivitive
				MeasureFlag = -1;					//Set the MeasureFlag.
				PDDBCntr = 0;						//Clear Peak Detect Dead Band Counter
				}
			else									//OTHERWISE,
				{
				if(MeasureFlag == -1)				//If the MeasureFlag has been previously set  (200us)
					{
					if(SampleCounter > 199)			//Set UPPER Limit if necessary
						X[Beats] = 200;
					else if(SampleCounter < 34)		//or the LOWER Limit. In all cases, SAVE
						X[Beats] = 33;
					else							//the value of the SampleCounter in the SMA filter array (X[]).
						X[Beats] = SampleCounter;

					//Calculate the RMS values of the AC for both RED and IR sources
					Rrms = sqrtf(RSum/(float)(SampleCounter-1));						//36us		
					IRrms = sqrtf(IRSum/(float)(SampleCounter-1));						//36us	

					//Calculate the Ratio R = (Rrms/Rdc) / (IRrms/IRdc)
					Rx = (Rrms/R_DC)/(IRrms/IR_DC);										//39us

					//Calculate the SPO2 from 3rd order fit polynomial
					SPO2 = ((10.002*Rx - 52.887)*Rx + 26.817)*Rx + 98.293;				//27us

					if(SPO2 > 100.)					//Clamp the SPO2 value
						SPO2 = 100.;
					SP2[Beats] = (int)SPO2;			//Store the calculated results in the SMA filter
					SampleCounter = 0;				//Set the Sample Counter back to -1.
					MeasureFlag = 0;				//Reset MeasureFlag
					RSum = 0;						//Clear the RMS accumulators
					IRSum = 0;

					if(Beats == MEDIAN_SIZE-1)		//Both SMA filters use Circular Buffers, setup endpointer
						SMAindex = 0;
					else
						SMAindex = Beats+1;

					//Perform Simple Moving Average (SMA) filtering on Heartbeat signal
					SMA = SMAM1 + (float)(X[Beats] - X[SMAindex])/(MEDIAN_SIZE-1);				//27us
					SMAM1 = SMA;

					//Perform Simple Moving Average (SMA) filtering on SPO2 signal
					SPO2ave = SPO2aveM1 + (float)(SP2[Beats] - SP2[SMAindex])/(MEDIAN_SIZE-1);	//27us
					SPO2aveM1 = SPO2ave;

					Beats++;						//Increment the SMA index
					if(Beats == MEDIAN_SIZE)		//If end of buffer reached, reset index.
						Beats = 0;
					PrintFlg = -1;					//Set Flag to output results.
					}
			if(PDDBCntr > PeakDetDB)
					{
	 			    Peak = Peak * DecayFactor;			//Calculate NEW Peak value for Peak Follower
					PDDBCntr = 0;
					}
			    if(Peak > 0.0) 						//Only -absolute value of peak allowed.
					Peak = 0.0;
				}
			PDDBCntr++;						//Increment Peak Detect Deadband Counter
			SampleCounter++;						//Increment Sample Counter

			RSum = RDiff*RDiff + RSum;				//Perform RMS Squared Sum for Red AC		//10us
			IRSum = IRDiff*IRDiff + IRSum;			//Perform RMS Squared Sum for IR AC			//10us

			GoodR = 0;								//Clear Flags
			GoodIR = 0;
		
			switch(DataSet)							//SW1 is used to select data output
				{
				case 0:
					PrintDat();						//Output IRac, dIRac and peak data to serial port each scan
					break;
				case 1:
					PrintIRDat();					//Output Red and IR waves with DC to serial port each scan.
					break;
				case 2:
					PrintResults();					//Output BPM and SPO2 to serial port each scan
					break;
				default:
					break;
				}					

			if(PrintFlg == -1)						//When the PrintFlg is SET
				{
				if(IRFreq[0] < 200)					//If IRFreq < 200
					{
					BPM = 0;						//Limit the BPM and SPO2ave to 0
					SPO2ave = 0;
					}
				else					
					BPM = (int)(60.0/0.010/SMA);	//Calculate the HeartRate from SMA output
				Write2LCD();						//Then, WRITE IT TO THE DISPLAY		//6.2ms (Fixed 25ms)
				PrintFlg = 0;						//Clear the Print Flag
				}
			dindex++;								//Increment counter
			for(timer=0;timer<100;timer++);
			}

		if(dindex == SampleSize)					//Is it time to tweek LED intensity?
			{
			STATE = -1;								//Set STATE counter back to -1
			dindex = 0;								//Initalized tweek counter
			DisableT1();							//Disable Timer T1
			SampleSize = ADJUST_PERIOD;				//Setup for next tweek.
			}
		}
	return 0;
	}


/*Subroutines*/
/*Procedure to display initial message to the operator*/
void Preamble(void)
	{
	clear();
	LCDstr[0] = 0;
	strcpy(LCDstr,"PulseOx ");
	LCDwrite(0,0,LCDstr);
	strcpy(LCDstr,"MiniBox ");
	LCDwrite(1,0,LCDstr);
	for(timer=0;timer<10000000;timer++);				
	clear();
	strcpy(LCDstr,"FIRMWARE");
	LCDwrite(0,0,LCDstr);
	strcpy(LCDstr,"2-NOV-08");
	LCDwrite(1,0,LCDstr);
	for(timer=0;timer<10000000;timer++);				
	clear();
	strcpy(LCDstr,"Insert  ");
	LCDwrite(0,0,LCDstr);
	strcpy(LCDstr,"Finger  ");
	LCDwrite(1,0,LCDstr);
	for(timer=0;timer<3000000;timer++);					
	clear();
	strcpy(LCDstr,"To Start");
	LCDwrite(0,0,LCDstr);
	strcpy(LCDstr,"Test... ");
	LCDwrite(1,0,LCDstr);
	for(timer=0;timer<3000000;timer++);					
	clear();
	}


/*Procedure to write the Average HeartRate (BPM) and the Average Blood Oxygen Saturation (SPO2)*/
/*to the LCD display.*/
void Write2LCD(void)
	{
	strcpy(LCDstr,"BPM: ");
	itoa(BPM,buf,10);
	strcat(LCDstr,buf);
	strcat(LCDstr,"  ");
	LCDwrite(0,0,LCDstr);						//3.12ms
	strcpy(LCDstr,"SP2: ");
	itoa((unsigned int)SPO2ave,buf,10);
	strcat(LCDstr,buf);
	strcat(LCDstr,"  ");
	LCDwrite(1,0,LCDstr);
	}



/*Procedure to send the following data out the serial communications port.      */
/*(IRDiff) - IR Difference between the IRFilt and the IR_DC signals             */
/*(5*dIRFilt) - Output of the High Pass Filter (Differentiator)                 */
/*(5*Peak) - Output from the Peak Follower Algorithm                            */
void PrintIRDat(void)
	{
//	sprintf(message,"%d  %d  %d\r\n",(int)IRDiff,(int)(5.*dIRFilt),(int)(5.*Peak));
	itoa((int)IRDiff,buf,10);
	strcpy(message,buf);
	strcat(message,"  ");
	itoa((int)(5.*dIRFilt),buf,10);
	strcat(message,buf);
	strcat(message,"  ");
	itoa((int)(5.*Peak),buf,10);
	strcat(message,buf);
	strcat(message,"\r\n");
	SendString(message);
	}



/*Procedure to send the following data out the serial communicaitons port       */
/*(RDat) - Buffered RFilt                                                       */
/*(R_DCDat) - Buffered DC signal from the RED Source                            */
/*(IRDat) - Buffered IRFilt                                                     */
/*(IR_DCDat) - Buffered DC signal from the IR Source                            */
void PrintDat(void)
	{
//	sprintf(message,"%u  %u  %u  %u\r\n",RDat,R_DCDat,IRDat,IR_DCDat);
	itoa(RDat,buf,10);
	strcpy(message,buf);
	strcat(message,"  ");
	itoa(R_DCDat,buf,10);
	strcat(message,buf);
	strcat(message,"  ");
	itoa(IRDat,buf,10);
	strcat(message,buf);
	strcat(message,"  ");
	itoa(IR_DCDat,buf,10);
	strcat(message,buf);
	strcat(message,"\r\n");		
	SendString(message);
	}



/*Procedure to send the following data out the serial communicaitons port       */
/* BPM - Heart Rate in Beats per Minute                                         */
/* SPO2ave - Oxygen Saturation in percent                                       */
void PrintResults(void)
	{
	itoa(BPM,buf,10);
	strcpy(message,buf);
	strcat(message,"  ");
	itoa(SPO2ave,buf,10);
	strcat(message,buf);
	strcat(message,"\r\n");		
	SendString(message);
	}



/*Routine to Set both LED Intensities via DAC, then adjust the IR source to have approximatly  */
/*the same brightness as the RED source. This procedure will take place as fast as the system  */
/*can perform the task.                                                                        */
void IDBrightLED(void)
	{
	DACOut(1,RDaqVal);									//Set RED Intensity		
	DACOut(0,IRDaqVal);									//Set IRED Intensity
	MeasurementDone = 0;
	LATB &= 0xFFFE;										//Turn OFF IRED Led
	LATB |= 0x2;										//Turn ON RED Led
	MEASURE = 0;										//Initialize STATE counter
	InitT2();											//Initialize Timer T2
	InitIC1();											//Initialized Input Capture 1
	while(!MeasurementDone);							//Wait till measurement completes
	itoa(RFreq[0],buf,10);								//Send the values IRFreq and RFreq
	strcpy(message,buf);								//out
	strcat(message,"  ");								//the
	itoa(IRFreq[0],buf,10);								//serial
	strcat(message,buf);								//port	
	strcat(message,"\r\n");								//for diagnostics
	SendString(message);
	}


/*Routine to tune up the IR LED intensity to match that of the RED source. Accomplishes this task*/
/*as fast as the system can handle.*/
void auto_adj(void)
	{
	ADJ_FLAG = -1;
	DACOut(0,IRDaqVal);									//Set IRED Intensity
	for(timer=0;timer<10;timer++);						//Delay a short spell.	
	while(ADJ_FLAG)
		{
		MeasurementDone = 0;							//Clear MeasurementDone Flag
		LATB &= 0xFFFE;									//Turn OFF IRED Led
		LATB |= 0x2;									//Turn ON RED Led
		MEASURE = 0;									//Initialize STATE counter
		InitT2();										//Initialize Timer T2
		InitIC1();										//Initialized Input Capture 1
		while(!MeasurementDone);						//Wait for measurement to complete
		if(IRFreq[0] > RFreq[0])						//If IR brighter than RED source,
			{
			if((IRFreq[0] - RFreq[0]) > BAND)			//If the IR intensity is greater than RED plus BAND,
				IRDaqVal -= STEPSIZE;					//Decrease the IR intensity by STEPSIZE
			else										//Otherwise,
				IRDaqVal--;								//Decrement the IR intensity,
			if(IRDaqVal < 1)							//Limit the IR intensity if necessary.
				IRDaqVal = 1;
			}
		else if(IRFreq[0] < RFreq[0])					//If the RED source is brighter than the IR source,
			{
			if((RFreq[0] - IRFreq[0]) > BAND)			//If the RED intensity is greater than the IR plus BAND,
				IRDaqVal += STEPSIZE;					//Increase the IR intensity by STEPSIZE,
			else										//Otherwise,
				IRDaqVal++;								//Increment the IR intensity.	
			if(IRDaqVal > 4094)							//Limit the IR intensity if necessary.
				IRDaqVal = 4094;
			}
		else											//If the intensities are close,
			ADJ_FLAG = 0;								//Clear the ADJ_FLAG.
		DACOut(0,IRDaqVal);								//Set IRED Intensity
		}
	}



/*Routine to setup SPI hardware*/
void SPIInit(void)
	{
	LATB |= 0x0004;										//Set *SS1=1;  (*CS = 1)
	SPI1STATbits.SPIEN = 1;								//Enable SPI module - SCK1, SDO1, SDI1, *SS1
	SPI1CON = 0x053E;									//Disable Framed, Enable 16 bit data width, Active
//	SPI1CON = 0x053D;									//Disable Framed, Enable 16 bit data width, Active
	}													//clock to Idle clock,active state hi,Master Mode
														//enabled,Secondary Prescaler /1, 
														//(E) Primary Prescaler /4 then SCK1 = 7.3728MHz  
														//(D) Primary Prescaler /16 then SCK1 = 1.8432MHz <--			
	
/*Routine to Write data out SPI port*/
void SPIOut(unsigned int data)
	{
	LATB &= 0xFFFB;			                    		//Set Chip Select LOW   (*CS = 0)
	SPI1BUF = data;										//Sent data out SPI Port
	T10us();											//Wait for 10 us									
	LATB |= 0x0004;										//Set Chip Select HIGH	(*CS = 1);
	}



/*Routine to write data to selected channel A or B   			*/
/*channel = 1:	RED DAC Command in <data>           			*/
/*        = 0:  IR  DAC Command in <data>						*/
void DACOut(char channel,unsigned int data)
	{
	if(channel)											//If channel = 1,
		SPIOut(data | 0x9000);							//Write RED DAC command,Output Enabled,2x gain.
	else												//If channel = 0,
		SPIOut(data | 0x1000);							//Write IRED DAC command,Output Enabled,2X gain
	}



/*Routine to reverse the characters in a string. There is only one argument for this procedure  */
/*The procedure is used only by the itoa procedure.                                             */
char *strrev(char *str)
	{
	char *p1, *p2;
	if (!str || !*str)
		return str;
	for (p1 = str, p2 = str + strlen(str) - 1; p2 > p1; ++p1, --p2)
		{
		*p1 ^= *p2;
		*p2 ^= *p1;
		*p1 ^= *p2;
		}
	return str;
	}



/*Routine to convert 16 bit signed/unsigned integer values to NULL terminated ASCII strings. */
/*   n - integer to be converted.                                                            */
/*  *s - pointer to string to be used as buffer.                    						 */
/*   b - base of ASCII conversion (usually base 10)											 */
/*The problem with this procedure is that the string is output in reverse fashion. The       */
/*characters are reversed by calling the strrev procedure after the conversion is complete.  */
char *itoa(int n, char *s, int b)
	{
	int i,sign;
    i=0;
	if((sign = n) < 0)
		n = -n;
	do 
		{
		s[i++] = digits[n % b];
		}
	while ((n /= b) > 0);
	if(sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	return strrev(s);
	}



/*Sets up the Serial Port UART hardware with the following characteristics  */
/*BAUDRATE:	115200															*/
/*DATASIZE:	8																*/
/*PARITY:	N																*/
/*STOPBIT:	1																*/
/*FLOW:		NONE															*/
void UARTinit(int BRGWord)
	{
 	TRISF = 0xC;										//Make pins U1RX and U1TX inputs
	U1BRG  = BRGWord;									//Calculate word for Baud Rate Register
	U1MODE = 0x8400; 									//Enable UART, Set 8-n-1, Use U1ARX/U1ATX 
	U1STA  = 0x0440;									//Reset status register and enable TX & RX
	_U1RXIF=0;											//Clear UART RX Interrupt Flag
	}



/*Send a single character out the UART*/
void SendChar(char data)
	{
    int i = 0;
	U1TXREG = data;										//Place data in transmit register.
		while ( !U1STAbits.TRMT && (i<10000) )			//Wait for byte to be sent or timeout.
			i++;
	}



/*Send a string out the UART*/
void SendString(char s[])
	{
	int i = 0;	
	//Send each character to trasmitter. Stop at NUL or largest string that can be sent.
	while(i < BigString) 
		{
		if( s[i] == '\0' ) 
			// quit on string terminator
			break; 
		SendChar(s[i++]);
		}
	}



/*Timer with value of 5us*/
void T5us(void)
	{
	for(TValueB = 0; TValueB < 17; TValueB++);
	}



/*Timer with value of 10us*/
void T10us(void)
	{
	for(TValueB = 0; TValueB < 34; TValueB++);
	}



/*Timer with value of 50us*/
void T50us(void)
	{
	for(TValueW = 0; TValueW < 161;TValueW++);
	}




/*Timer with value of 100us*/
void T100us(void)
	{
	for(TValueW = 0; TValueW < 322;TValueW++);
	}


/*Timer with value of 1ms*/
void T1ms(void)
	{
	for(TValueW = 0; TValueW < 3220;TValueW++);
	}



/*Timer with value of 5ms*/
void T5ms(void)
		{
		for (TValue = 0;TValue < 51;TValue++)
			T100us();
		}



/*Timer with value of 15ms*/
void T15ms(void)
		{
		for (TValue = 0;TValue < 151;TValue++)
			T100us();
		}


/*Initialize Display for 4 bit interface.*/
void Initdisp(void)
	{
	LATF &= 0xFFEF;										//Drop Enable Pin low (E = 0) 
	LATF &= 0xFFDF;										//Reset RS line low (RS = 0)
	LATB &= 0xFFF7;										//Reset R/*W line LOW (write) (R/*W = 0)
	T15ms();											//Wait for 15ms
	LATB |= 0x30;										//Set function for 8 bit
	toggleE();											//Toggle E bit
	T5ms();												//Wait for more than 4.1ms
	toggleE();											//Repeat, then 
	T100us();											//Wait an additional 100us
	toggleE();											//Repeat.
	LATB = (LATB & 0xFF0F) | 0x20;						//Set function for 4 bit interface
	toggleE();											//Toggle E bit
	toggleE();											//Repeat function for 4 bit interface HI Nibble
	LATB = (LATB & 0xFF0F) | 0x80;						//Second Nibble (2 lines)
	toggleE();											//Toggle E bit
	LATB &= 0x0F;										//Turn OFF display
	toggleE();											//Toggle E bit
	LATB = (LATB & 0xFF0F) | 0xE0;
	toggleE();											//Toggle E bit
	LATB &= 0x0F;										//Clear Display
	toggleE();											//Toggle E bit
	LATB = (LATB & 0xFF0F) | 0x10;
	toggleE();											//Toggle E bit
	LATB &= 0xFF0F;										//Set Entry Mode (Increment)
	toggleE();											//Toggle E bit
	LATB = (LATB & 0xFF0F) | 0x60;
	toggleE();											//Toggle E bit
	LATB |= 0x8;										//Set R/*W line HI (write) (R/*W = 1)
	}



/*Routine to apply a 100us pulse to the E input of the display.*/
void toggleE(void)
	{
	LATF |= 0x0010;										//Set E pin (E = 1)
	T50us();											//100 us Delay
	LATF &= 0xFFEF;										//Reset E pin (E = 0)
	}



/*Routine to load CONTROL character into the display (4 bit bus).*/
void DispCmd(unsigned char data)
	{
	LATB &= 0xFFF7;										//Reset R/*W line LOW (write) (R/*W = 0)
	LATF &= 0xFFDF;										//Reset RS line low (RS = 0)
	LATB = (LATB & 0xFF0F) | (data & 0xF0);				//Load Upper Nibble of data
	toggleE();											//Toggle E-Line
	LATB = (LATB & 0xFF0F) | ((data << 4) & 0xF0);		//Load Lower Nibble of data
	toggleE();											//Toggle E-Line
	LATB |= 0x0008;										//Set R/*W line HI (write) (R/*W = 1)
	T1ms();												//Wait 5ms
	}



/*Routine to load DATA characters into the display (4 bit bus).*/
void DispDat(unsigned char data)
	{
	LATB &= 0xFFF7;										//Reset R/*W line LOW (write) (R/*W = 0)
	LATF |= 0x0020;										//Set RS line high (RS = 1)
	LATB = (LATB & 0xFF0F) | (data & 0xF0);				//Load Upper Nibble of data
	toggleE();											//Toggle E-Line
	LATB = (LATB & 0xFF0F) | ((data << 4) & 0xF0);		//Load Lower Nibble of data
	toggleE();											//Toggle E-Line
	LATB |= 0x0008;										//Set R/*W line HI (write) (R/*W = 1)
	}



/*Routine to display a NULL terminated string located at pointer position str on LCD at a       */
/*location defined by row (r) and column (c)                                                    */
/* r - row to locate data string. values: {0,1}													*/
/* c - column to locate data string. values: [0-7}												*/
/* *str - pointer to NULL terminated string.													*/
void LCDwrite(char r, char c, char *str)
	{
	cursor(r,c);									//Place cursor at position {r,c}    			//1.08ms
	while(*str != 0)									//Continue until NULL character is reached.
		{
		DispDat(*str++);								//Display the character pointed to by str. //200us
		}
	curOFF();											//Turn the cursor OFF if it has been left on.
	}



/*Routine to place the cursor at the specified position {row,column}    */
/* r - row to place cursor. Values: {0,1}								*/
/* c - column to place cursor. Values: {0-7)							*/
void cursor(unsigned char row, unsigned char column)
	{
	DispCmd((unsigned char) ((row*0x40 + column) | 0x80));
	}



/*Routine to turn OFF the cursor*/
void curOFF(void)
	{
	DispCmd(0xC);
	}



/*Routine to turn ON cursor*/
void curON(void)
	{
	DispCmd(0xE);
	}



/*Routine to CLEAR display*/
void clear(void)
	{
	DispCmd(0x01);
	curOFF();
	}



/*Routine to turn blinking ON*/
void blinkON(void)
	{
	DispCmd(0x0D);
	}



/*Routine to turn blinking OFFr*/
void blinkOFF(void)
	{
	DispCmd(0x0C);
	}



/*Routine to Initialized timer T1												*/
/*The timer is setup to use the internal clock FCY with prescaler = 256			*/
/*In this application: 	FCY = XTFREQ*PLLMODE/4		 							*/
/*								where, 	PLLMODE = 16 							*/
/*										XTFREQ = 7372800 HZ						*/
/*										FCY = 29,491,200						*/
/*The timer has a period set so that after 5ms, an interrupt occurs.			*/
void InitT1(void)
	{
	TMR1 = 0;											// Clear Timer 1
	PR1 = 0x0240;										// interrupt every 5ms
//	PR1 = 0x0120;										// interrupt every 2.5ms (experimental)
//	PR1 = 0x73;											// interrupt every 1ms	 (experimental)
	IFS0bits.T1IF = 0;									// clr interrupt flag
	IEC0bits.T1IE = 1;									// set interrupt enable bit
	T1CON = 0x0030;										// Fosc/4, 1:256 prescale, DO NOT start TMR1 YET
	}									



/*Routine to DISABLE timer T1*/
void DisableT1(void)
	{	
	T1CONbits.TON = 0;
	}



/*Routine to ENABLE timer T1*/
void EnableT1(void)
	{
	T1CONbits.TON = 1;
	}



/*Routine to Initialzed and start the timer T2 with an internal clock set to FCY	*/
/*In this application: 	FCY = XTFREQ*PLLMODE/4		 								*/
/*								where, 	PLLMODE = 16 								*/
/*										XTFREQ = 7372800 HZ							*/
/*										FCY = 29,491,200							*/
void InitT2(void)					
	{
	TMR2 = 0;											//Clear timer 2
	T2CON = 0x8000;										//Fosc/4, 1:1 prescale, start TMR2	
	}



/*Routine to DISABLE timer T2*/
void DisableT2(void)
	{	
	T2CONbits.TON = 0;
	}



/*Routine to ENABLE timer T2*/
void EnableT2(void)
	{
	T2CONbits.TON = 1;
	}



/*Initialize Input Capture Hardware (IC1). Sets up to trigger interrupt on every	*/
/*rising edge at IC1 input pin. The routine also clears the hardware by reading the */
/*buffer                                                                            */
void InitIC1(void)
	{
	IFS0bits.IC1IF = 0;									//clr interrupt flag
	IEC0bits.IC1IE = 1;									//set interrupt enable bit
	IC1CON = 0x0083;									//TMR2 captured on every rising edge of IC1
	}



/*Routine to DISABLE Capture IC1*/
void DisableIC1(void)
	{	
	IEC0bits.IC1IE = 0;
	IC1CON = 0;											//Disable IC1
	}



/*Interrupt routine to setup the basic timing for the system. The routine is called */
/*when timer T1 expires. This happens every 10ms. A state counter called STATE is   */
/*used to sequence the necessary events that the routine handle. Each state is 10ms */
/*Since these STATES are within an interrrupt routine, the first STATE (-1) is      */
/*is executed in the MAIN routine. STATE -1, has the timer initialization code.		*/
/*STATE -1:	> Turns OFF both LEDS													*/
/*			> Sets Initial DAC values												*/
/*			> Initialize Timer T1 													*/
/*STATE 0:  > Set PULSE output = 0                                                  */
/*			> set measure = 0                                                       */
/*          > initialized timer T2                                                  */               
/*          > initialized IC1                                                       */
/*STATE 1:	> Set PULSE output = 1													*/
/*			> Perform filtering														*/
/*			> Perform DC Tracking													*/
/*			> Set New DAC values													*/
void __attribute__((interrupt)) _T1Interrupt(void)
	{
	IFS0bits.T1IF = 0;									//Clear interrupt flag	

	if(STATE == 0)										//STATE: 0
		{
//		LATB &= 0xFDFF;									//Turn OFF diagnostic bit 9 (Measure Filter time)
		LATB &= 0xFFFE;									//Turn OFF IRED Led (VIRon = 0)
		LATB |= 0x2;									//Turn ON RED Led (VRon = 1)
		MEASURE = 0;									//set state counter - Measurement: 0
		STATE++;										//Increment state counter - STATE++
		InitT2();										//Initialize Timer T2
		InitIC1();										//Initialized Input Capture 1
		}
	else if(STATE == 1)									//STATE: 1							
		{
//		LATB |= 0x200;									//Turn ON diagnostic bit 9 (Measure Filter time)

		//LPFilter Algorithm (Red Source)
		RFilt[0] = (N0*RFreq[0] + N1*RFreq[1] + N2*RFreq[2] + N3*RFreq[3] + N4*RFreq[4] - D1*RFilt[1] - D2*RFilt[2] - D3*RFilt[3] - D4*RFilt[4])/D0; 
		RFilti = (unsigned int)RFilt[0];				//Buffer RFilt signal

		//HPFilter Algorithm or Differentiator (RED Source)
		dRFilt = K1*(RFilt[0] - RFilt[1]) + K2*dRFiltM1;

		for(indx = 4; indx > 0;indx--)					//Shift RED samples for LP Filter
			{
			RFreq[indx] = RFreq[indx-1];
			RFilt[indx] = RFilt[indx-1];
			}
		dRFiltM1 = dRFilt;								//Shift RED samples for HP Filter

		//DC Filter Algorithm
		R_DC = R_DCM1 + (RFreq[0] - R_DCM1)/K;
		R_DCM1 = R_DC;									//Shift RED samples for DC Filter
		R_DCi = (unsigned int)R_DC;						//Buffer R_DC signal

		RDiff = RFilt[0] - R_DC;						//Calculate RED AC signal
		GoodR = -1;										//Good Data available



		//LPFilter Algorithm (IR Source)
		IRFilt[0] = (N0*IRFreq[0] + N1*IRFreq[1] + N2*IRFreq[2] + N3*IRFreq[3] + N4*IRFreq[4] - D1*IRFilt[1] - D2*IRFilt[2] - D3*IRFilt[3] - D4*IRFilt[4])/D0; 
 		IRFilti = (unsigned int)IRFilt[0];				//Buffer IRFilt signal

		//HPFilter Algorithm or Differentiator (IR Source)
		dIRFilt = 5.*K1*(IRFilt[0] - IRFilt[1]) + K2*dIRFiltM1;

		for(indx = 4; indx > 0;indx--)					//Shift IR samples for LP Filter
			{
			IRFreq[indx] = IRFreq[indx-1];
			IRFilt[indx] = IRFilt[indx-1];
			}
		dIRFiltM1 = dIRFilt;							//Shift IR samples for HP Filter

		//DC Filter Algorithm
		IR_DC = IR_DCM1 + (IRFreq[0] - IR_DCM1)/K;
		IR_DCM1 = IR_DC;								//Shift IR samples for DC Filter
		IR_DCi = (unsigned int)IR_DC;					//Buffer IR_DC signal

		IRDiff = IRFilt[0] - IR_DC;						//Calculate IR AC signal
		GoodIR = -1;									//Good Data available

//		LATB &= 0xFDFF;									//Turn OFF diagnostic bit 9 (Measure Filter time)

//		DACOut(0,IRDaqVal);								//Set IRED Intensity
//		DACOut(1,RDaqVal);								//Set RED Intensity		
		STATE--;										//Decrement STATE COUNTER
		}
	}



/*Interrupt routine to measure the intensity of the LEDs by measuring the frequency	*/
/*being fed back from the TAOS LTF device. The routine is invoked by the IC1 input  */
/*pin (Input Capture #1). The hardware is setup so that a rising edge on the IC1 pin*/
/*causes the interrupt to occur. There four measurement states within this routine. */
/*The state counter MEASURE is used to control the events.                          */
/*MEASURE: 0	>  Read IC1 buffer and save	(T0)									*/
/*				>  Increment state counter MEASURE									*/
/*MEASURE: 1	>  Read IC1 buffer again and calculate the period for the RED Source*/
/*				   Actual Period is: FCY/RFreq[0]. 								 	*/
/*				>  Compute RED Intensity - Directly proportional to frequency		*/ 
/*				>  Turn OFF the RED Led												*/
/*				>  Turn ON the IR Led												*/
/*				>  Increment state counter MEASURE									*/
/*MEASURE: 2	>  Read IC1 buffer and save	(T0)									*/
/*				>  Increment state counter MEASURE									*/	
/*MEASURE: 3	>  Read IC1 buffer again and calculate the period for the IR Source	*/
/*				   Actual Peioud is: FCY/IRFreq[0].									*/
/*				>  Compute IR Intensity - Directly proportional to frequency		*/ 
/*				>  Turn OFF the IRED Led											*/
/*				>  Set the state counter MEASURE = 0								*/
/*				>  Disable the INPUT CAPTURE hardware								*/
/*				>  Disable the COUNTER T2											*/
void __attribute__((interrupt)) _IC1Interrupt(void)
	{
	IFS0bits.IC1IF = 0;									//Clear interrupt flag	
	if(MEASURE == 0)									//MEASUREMENT STATE: 0
		{
		T0 = IC1BUF;									//Save Initial Timer Value (RED Source)
		MEASURE++;										//Increment State Counter
		}
	else if(MEASURE == 1)								//MEASUREMENT STATE: 1
		{
		RPeriod = IC1BUF - T0;							//Calculate Period for RED Source
//		RFreq[0] = 589824/RPeriod;						//Calculate RED Intensity (Proportional to Frequency)
		RFreq[0] = (unsigned int)((long)1179648/(long)RPeriod);					//Calculate RED Intensity (Proportional to Frequency)
		if(RPeriod < 200)								//If Period too small, 
			RFreq[0] = 0;								//Set Intensity to ZERO
		LATB &= 0xFFFD;									//Turn OFF RED Led
		LATB |= 0x1;									//Turn ON IRED Led
		MEASURE++;
		}
	else if(MEASURE == 2)								//MEASURE STATE: 2
		{
		T0 = IC1BUF;									//Save Initial Timer Value (IR Source)	
		MEASURE++;										//Increment State Counter
		}
	else if(MEASURE == 3)								//MEASUREMENT STATE: 3
		{
		IRPeriod = IC1BUF - T0;							//Calculate Period for IR Source
//		IRFreq[0] = 589824/IRPeriod;					//Calculate IR Intensity (Proportional to Frequency)
		IRFreq[0] = (unsigned int)((long)1179648/(long)IRPeriod);				//Calculate IR Intensity (Proportional to Frequency)
		if(IRPeriod < 200)								//If Period too small, 
			IRFreq[0] = 0;								//Set Intensity to ZERO
		LATB &= 0xFFFE;									//Turn OFF IRED Led
		MEASURE = 0;									//Set MEASURE STATE to 0
		IC1CON = 0;										//Turn OFF IC1
		IEC0bits.IC1IE = 0;								//Disable IC1 Interrupt
		T2CON = 0;										//Disable Timer T2
		MeasurementDone = -1;							//Set Measurement Flag
		}
	}



//Diagnostic Scope Markers (Use PB9 - Pin 23)
//LATB |= 0x200;									//Turn ON diagnostic bit 9
//LATB &= 0xFDFF;									//Turn OFF diagnostic bit 9
